////////////////////////////////////////////////////////////////////////////
// Fragment shader program to compute per-face normals with arbitrary 
// vertex valency.
//
// Copyright (C) 2007 Harlen Costa Batagelo. All rights reserved.
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the 
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Global variables
////////////////////////////////////////////////////////////////////////////

// Size of a dimension of the vertex attribute map, in texels.
uniform int g_iVtMapSize; 

uniform sampler2D s0; // Face ID texture
uniform sampler2D s1; // Vertex position texture

////////////////////////////////////////////////////////////////////////////
// Convert 1D index (i) to 2D UV index.
// i = 1D index
// s = size of a screen line, in pixels
////////////////////////////////////////////////////////////////////////////
vec2 I1DTo2D(float i, float s) { return vec2(mod(i,s),int(i/s)); }

////////////////////////////////////////////////////////////////////////////
// Pixel shader used for computing face normals.
////////////////////////////////////////////////////////////////////////////
void main( )
{
	// Sample face ID texture
	vec3 fVtIdx = vec3( texture2D( s0, gl_TexCoord[0].st ).xyz );
	if( fVtIdx.x == -1.0 ) discard;
	
	// Sample vertex position
	vec2 fVt1UV = I1DTo2D( fVtIdx.x, float(g_iVtMapSize) );
	vec2 fVt2UV = I1DTo2D( fVtIdx.y, float(g_iVtMapSize) );
	vec2 fVt3UV = I1DTo2D( fVtIdx.z, float(g_iVtMapSize) );	
    vec3 vV1 = vec3( texture2D( s1, fVt1UV ).xyz );
    vec3 vV2 = vec3( texture2D( s1, fVt2UV ).xyz );
    vec3 vV3 = vec3( texture2D( s1, fVt3UV ).xyz );
    
	vec3 vDV1 = gl_Normal.xyz;//vV2 - vV1;
	vec3 vDV2 = vV3 - vV1;
	
    gl_FragColor = vec4( cross( vDV1, vDV2 ), 1.0 );
}
